/*	========================================================
*	- NAME:
*	  + Railgun Blast
*
*	- DESCRIPTION:
*	  + This is a fun plugin that adds a 'conc blast'
*	  + to the hit point of the railgun shot.
*	  + When the shot hits a wall/player/etc it will
*	  + create a 'conc blast' and bounce anyone
*	  + within the radius a certain distance away
*	  + depending on how close they are to the explosion.
*
*	- CREDITS:
*	  + The old TF-Tech mod for giving me the idea.
*	  + TKG for helping me create a new push function.
*	  + Ramono for letting me know to register the entity with a touch.
*
*	-------------
*	Server cvars:
*	-------------
*	- railgunbounce <0/1/2>
*	  + Setting to 0 will turn the railgun blast off.
*	  + Setting to 1 will turn the railgun blast on and bounce everyone.
*	  + Setting to 2 (default) will turn the railgun blast on and bounce only you.
*
*	---------
*	Versions:
*	---------
*	2.1 - Remade the push function, thanks TKG! (10-28-2011).
*	2.0 - Remade the plugin to be more optimized (10-27-2011).
*	1.1 - Added the cvar to bounce only yourself (08-30-2007).
*	1.0 - First version made and works (08-21-2007).
*/

#include <amxmodx>
#include <fakemeta>
#include <tfcx>
#include <fun>

#pragma semicolon 1

new const PLUGIN[] = "Railgun Blast";
new const VERSION[] = "2.1";
new const AUTHOR[] = "hlstriker";

#define set_as_rail(%1) set_pev(%1, pev_iuser1, 1)
#define is_ent_rail(%1) pev(%1, pev_iuser1)
#define unset_rail(%1) set_pev(%1, pev_iuser1, 0)

new cvar_railgunbounce;
new g_iMaxPlayers;
new g_iModelIndex_RpgRocket;
const EVENT_CONC_EXPLODE = 30;
const MAX_DISTANCE_FROM_RAIL = 170;
const Float:MIN_RAIL_PUSH_SPEED = 625.0;
const Float:MAX_RAIL_PUSH_SPEED = 1800.0;

public plugin_init()
{
	register_plugin(PLUGIN, VERSION, AUTHOR);
	g_iMaxPlayers = get_maxplayers();
	cvar_railgunbounce = register_cvar("railgunbounce", "2");
	register_forward(FM_SetModel, "hook_SetModel_Post", 1);
	register_forward(FM_Touch, "hook_Touch");
	register_event("ResetHUD", "hook_ResetHUD", "be");
}

public plugin_precache()
{
	g_iModelIndex_RpgRocket = precache_model("models/rpgrocket.mdl");
}

public hook_ResetHUD(iClient)
{
	if(get_pcvar_num(cvar_railgunbounce) == 0)
		return;
	
	set_task(0.4, "task_Spawn", iClient);
}

public task_Spawn(iClient)
{
	if(is_user_alive(iClient) && pev(iClient, pev_playerclass) == 10)
	{
		give_item(iClient, "tf_weapon_railgun");
		engclient_cmd(iClient, "use", "tf_weapon_railgun");
	}
}

public hook_SetModel_Post(iEnt, const szModel[])
{
	if(get_pcvar_num(cvar_railgunbounce) == 0)
		return;
	
	if(!pev_valid(iEnt))
		return;
	
	if(g_iModelIndex_RpgRocket != pev(iEnt, pev_modelindex))
		return;
	
	static iOwner;
	iOwner = pev(iEnt, pev_owner);
	if(!IsPlayer(iOwner))
		return;
	
	// Only allow engineers and random to use it.
	if(!(9 <= pev(iOwner, pev_playerclass) <= 10))
		return;
	
	set_as_rail(iEnt);
	tfc_setweaponbammo(iOwner, TFC_WPN_RAILGUN, 51);
}

public hook_Touch(iTouched, iToucher)
{
	if(get_pcvar_num(cvar_railgunbounce) == 0)
		return;
	
	if(!pev_valid(iTouched))
		return;
	
	if(g_iModelIndex_RpgRocket != pev(iTouched, pev_modelindex))
		return;
	
	if(!is_ent_rail(iTouched))
		return;
	
	static iOwner;
	iOwner = pev(iTouched, pev_owner);
	if(!IsPlayer(iOwner))
		return;
	
	unset_rail(iTouched);
	CreateBlast(iTouched, iOwner);
}

CreateBlast(iEnt, iOwner)
{
	// Make the shockwave.
	static Float:fRailOrigin[3];
	pev(iEnt, pev_origin, fRailOrigin);
	engfunc(EngFunc_PlaybackEvent, 0, iEnt, EVENT_CONC_EXPLODE, 0.0, fRailOrigin, Float:{0.0,0.0,0.0}, 0.0, 0.0, 0, 0, 0, 0);
	
	// Push owner.
	DoPush(iOwner, fRailOrigin);
	
	// Check to see if we should push others.
	if(get_pcvar_num(cvar_railgunbounce) == 1)
	{
		// Push everyone except owner (since they already got pushed).
		static iClient;
		for(iClient=1; iClient<=g_iMaxPlayers; iClient++)
		{
			if(!is_user_alive(iClient) || iOwner == iClient)
				continue;
			
			DoPush(iClient, fRailOrigin);
		}
	}
}

DoPush(iClient, Float:fRailOrigin[3])
{
	static Float:fClientOrigin[3], Float:fDistance;
	pev(iClient, pev_origin, fClientOrigin);
	
	fDistance = get_distance_f(fClientOrigin, fRailOrigin);
	if(fDistance > MAX_DISTANCE_FROM_RAIL)
		return;
	
	SetVelocityFromOrigin(iClient, fClientOrigin, fRailOrigin, (MAX_RAIL_PUSH_SPEED * ((MAX_DISTANCE_FROM_RAIL - fDistance) / MAX_DISTANCE_FROM_RAIL)));
}

SetVelocityFromOrigin(iClient, Float:fClientOrigin[3], Float:fRailOrigin[3], Float:fSpeed)
{
	static Float:fVelocity[3];
	fVelocity[0] = fClientOrigin[0] - fRailOrigin[0];
	fVelocity[1] = fClientOrigin[1] - fRailOrigin[1];
	fVelocity[2] = fClientOrigin[2] - fRailOrigin[2];
	
	vector_to_angle(fVelocity, fVelocity);
	angle_vector(fVelocity, ANGLEVECTOR_FORWARD, fVelocity);
	
	if(fSpeed < MIN_RAIL_PUSH_SPEED)
		fSpeed = MIN_RAIL_PUSH_SPEED;
	
	fVelocity[0] *= fSpeed;
	fVelocity[1] *= fSpeed;
	fVelocity[2] *= fSpeed;
	fVelocity[2] = fVelocity[2] * -1;
	
	static Float:fOldVelocity[3];
	pev(iClient, pev_velocity, fOldVelocity);
	fVelocity[0] += (fOldVelocity[0] / 1.5);
	fVelocity[1] += (fOldVelocity[1] / 1.5);
	fVelocity[2] += (fOldVelocity[2] / 4);
	
	set_pev(iClient, pev_velocity, fVelocity);
}

bool:IsPlayer(const iEnt)
{
	if(1 <= iEnt <= g_iMaxPlayers)
		return true;
	return false;
}